from genanki import Model

CHINESE_CARD_MODEL = Model(
    1559383145,
    'autoanki-chinese',
    fields=[
        {
            'name': 'word',
            'font': 'Arial',
        },
        {
            'name': 'word_traditional',
            'font': 'Arial',
        },
        {
            'name': 'pinyin',
            'font': 'Arial',
        },
        {
            'name': 'definition',
            'font': 'Arial',
        },
    ],
    templates=[
        {
            'name': 'Card 1',
            'qfmt': '{{word}}\n[{{word_traditional}}]',
            'afmt': '{{FrontSide}}\n\n<hr id=answer>\n{{pinyin}}<br>\n{{definition}}',
        },
        # {
        # 'name': 'Card 2',
        # 'qfmt': '{{FrontSide}}\n\n<hr id=answer>\n{{Pinyin}}\n{{Back}}',
        # 'afmt': '{{Front}}',
        # },
    ],
    css='.card {\n font-family: arial;\n font-size: 30px;\n text-align: center;\n color: black;\n background-color: white;\n}\n',
)

# # All the resources for finding the
#         CHINESE_CARD_MODEL = Model(
#             1559383145,
#             'Chinese Card (autoanki)',
#             fields=[
#                 {
#                     'name': 'word',
#                     'font': 'Arial',
#                 },
#                 {
#                     'name': 'word_traditional',
#                     'font': 'Arial',
#                 },
#                 {
#                     'name': 'pinyin',
#                     'font': 'Arial',
#                 },
#                 {
#                     'name': 'definition',
#                     'font': 'Arial',
#                 },
#             ],
#             templates=[
#                 {
#                     'name': 'Card 1',
#                     'qfmt': '{{word}}\n[{{word_traditional}}]',
#                     'afmt': '{{FrontSide}}\n\n<hr id=answer>\n{{pinyin}}<br>\n{{definition}}',
#                 },
#                 # {
#                 # 'name': 'Card 2',
#                 # 'qfmt': '{{FrontSide}}\n\n<hr id=answer>\n{{Pinyin}}\n{{Back}}',
#                 # 'afmt': '{{Front}}',
#                 # },
#             ],
#             css='.card {\n font-family: arial;\n font-size: 30px;\n text-align: center;\n color: black;\n background-color: white;\n}\n',
#         )
#         my_note = genanki.Note(
#             model=CHINESE_CARD_MODEL,
#             fields=['Capital of Canada', 'Ottawa'],
#             sort_field=1
#         )
#         my_deck = genanki.Deck(
#             2023480110,
#             'Country Capitals'
#         )
#
#         my_deck.add_note(my_note)
#         genanki.Package(my_deck).write_to_file('output.apkg')




from os import path
import logging

import pinyin
import jieba
import jieba.posseg as pseg
import chinese_converter
from wordfreq import word_frequency

from autoanki.Dictionary.Dictionary import Dictionary

PATH_TO_FILE = path.join(path.dirname(__file__), 'cedict_ts.u8')


# Use the lookup table here: https://github.com/fxsjy/jieba
NOUN = ['n', 'nt', 'nrt', 'ORG', 'nr', 'PER', 'ns', 'nw', 'nz', 'f', 's', 'an', 'zg', 'j']
TIME = ['t', 'TIME', 'tg', 'g']
ADPOSITION = ['p']
PRONOUN = ['r']
QUANTIFIER = ['q', 'm']
VERB = ['v', 'vd', 'vn', 'e', 'vg', 'h']
ADVERB = ['ad', 'd', 'aq']
OTHER = ['vn', 'xc', 'x']
ADJECTIVE = ['a', 'b', 'ng']
CONJUNCTION = ['c']
PARTICLE = ['y', 'u', 'uj', 'uz', 'ul', 'uv', 'ud', 'ug']
PUNCTUATION = ['w']
IDIOM = ['i', 'l']
SUFFIX = ['k']
OTHER = ['z', 'o', 'x']


class CEDictionary(Dictionary):

    def __init__(self, debug_level):
        """Chinese to English Dictionary
        Parser for the CC-CEDICT Dictionary:
            https://www.mdbg.net/chinese/dictionary?page=cedict
        This is a file containing 122,839 entries
        """
        self.logger = logging.getLogger('autoanki.cedict') 
        self.logger.setLevel(debug_level)
        self.logger.info("Loading Chinese Dictionary (CEDictionary)")
        self.dict = self._parse_file()
        self.logger.debug("Done!")
        
        pass

    def _parse_file(self) -> dict[str, dict]:
        self.logger.debug("Parsing file...")
        if not path.isfile(PATH_TO_FILE):
            self.logger.critical("Could not open dictionary file")

        # TODO Some entries use a dot in the midde
        #   e.g. 哈利·波特
        #   this does not currently match
        definitions = {}
        last_word = "~"
        current_word = ""

        with open(PATH_TO_FILE, "r") as dict_file:
            for line in dict_file:
                if line[0] == '#':
                    continue
                parts = line.split(" ")
                trad_word = parts[0]
                current_word = parts[1]

                definition = line[line.find("]")+2:]

                if current_word != last_word:
                    # self.logger.debug(f"Creating entry: {current_word}")
                    definitions[current_word] = {
                        'trad_word': "", 
                        'pinyin_numbers':"", 
                        'definition':"", 
                        'pinyin':"",
                        'frequency':"",
                    }
                
                # These only need to be done the first time the word is seen
                if last_word != current_word:
                    definitions[current_word]['trad_word'] = trad_word
                    definitions[current_word]['pinyin'] = pinyin.get(current_word)
                    definitions[current_word]['pinyin_numbers'] = pinyin.get(current_word, format="numerical")
                    definitions[current_word]['frequency'] = str(word_frequency(current_word, 'zh'))

                definitions[current_word]['definition'] += '<br>' + definition 
                last_word = current_word

        for key, _ in definitions.items():
            definitions[key]['definition'].strip('\n')
        return definitions


    def find_word(self, word: str) -> None | list[str]:
        """
        Find a word in the dictionary. This can be simplified, or traditional
        `return` Paramaters that get passed to the database
        """
        # Convert the word to simplified if needed
        word = chinese_converter.to_simplified(word)
        # self.logger.info(f"[{word}] [{word in self.dict}]")
        if not word in self.dict:
            return None

        """
        traditional_script = params[0]\n
        word_type = params[1]\n
        pinyin = params[2]\n
        pinyin_numbers = params[3]\n
        sub_components = params[4]\n
        hsk_level = params[5]\n
        top_level = params[6]\n
        definition = params[7]\n
        frequency = params[8]
        word = params[9]
        """
        params = ["", "", "", "", "", "", "", "", "", ""]

        slice = next(pseg.cut(word))
        part_of_speech = self.get_part_of_speech(word, slice.flag)
        if part_of_speech != None:
            params[1] = part_of_speech

        params[0] = self.dict[word]['trad_word'] 
        params[2] = self.dict[word]['pinyin'] 
        params[3] = self.dict[word]['pinyin_numbers'] 
        params[7] = self.dict[word]['definition'] 
        # self.logger.debug(f"Word: {word}")
        params[8] = self.dict[word]['frequency'] 
        params[9] = word
        return params

    def size(self):
        with open(PATH_TO_FILE,"r") as f:
            return len(f.readlines()) 

    def get_part_of_speech(self, word:str, code:str) -> str:
        if code in NOUN:
            return "noun"
        if code in TIME:
            return "time"
        if code in ADPOSITION:
            return "adposition"
        if code in PRONOUN:
            return "pronoun"
        if code in QUANTIFIER:
            return "quantifier"
        if code in VERB:
            return "verb"
        if code in ADVERB:
            return "adverb"
        if code in OTHER:
            return "other"
        if code in ADJECTIVE:
            return "adjective"
        if code in CONJUNCTION:
            return "conjunction"
        if code in PARTICLE:
            return "particle"
        if code in PUNCTUATION:
            return "punctuation"
        if code in IDIOM:
            return "idiom"
        if code in PARTICLE:
            return "particle"
        if code in OTHER:
            return "other"
        if code in SUFFIX:
            return "suffix"

        self.logger.warning(f"Part of speech code not covered: {word}:{code}")
        return "other"






import logging

import genanki
from wordfreq import word_frequency

from autoanki.DeckManager import template_decks

class DeckManager:
    """
    The class to make anki decks. Create the file using generate_deck_file()
    One of the most important concepts is the id. No matter what deck the word is in, it should have the same id so the
    same card in different decks can be remembered.
    This class makes extensive use of genanki, so understanding how genanki works
        is pretty significant for understanding this.
    """

    def __init__(self, debug_level,
                include_traditional=True,
                 include_part_of_speech=True
                 ):
        self.logger = logging.getLogger('autoanki.dckmngr')
        self.logger.setLevel(debug_level)
        self.deck = genanki.Deck(
            2020000110,
            "autoankiTesting"
        )
        self.include_pinyin_numbers = False
        self.include_number_of_strokes = False
        self.include_traditional = include_traditional 
        self.include_part_of_speech = include_part_of_speech

        self.book_list = []

    def settings(self,
                 include_traditional,
                 include_part_of_speech,
                 word_frequency_filter
                 ):
        """
        Configures settings for what's in the deck, and how it looks
        """
        self.include_traditional = include_traditional
        self.include_part_of_speech = include_part_of_speech 
        self.word_frequency_filter = word_frequency_filter 

    def generate_deck_file(self, words, deck_name: str, filename: str):
        """
        Generates a deck file from the database
        :param deck_name: The name of the deck to be created
        :param definitions_filename: The name of the file containing the definitions.
        :return:
        """

        # Number of valid cards that have been added to the deck
        num_of_valid_cards_added = 0

        # length = general_functions.file_len(definitions_filename)
        # self.deck.add_note()
        self.deck = genanki.Deck(
            2020000110,
            deck_name
        )

        for row in words:

            if self.word_frequency_filter:
                if self.word_frequency_filter < row['frequency']:
                    continue

            word = row["word"]

            if self.include_traditional:
                word_traditional = row["word_traditional"]
            else:
                word_traditional = ""

            if len(word) != len(word_traditional):
                self.logger.error(f"{word} and {word_traditional} should be the same length")

            # Replace all matching characters with a dash 
            formatted_word_traditional = ""
            for i in range(len(word)):
                if word[i] == word_traditional[i]:
                    formatted_word_traditional += "-"
                else:
                    formatted_word_traditional += word_traditional[i]

            if row["pinyin"]:
                pinyin = row["pinyin"]
            else:
                pinyin = row["pinyin_numbers"]
            definition = "<br>" + row["definition"]
            # word["word_id"]
            # word["word"]
            # word["word_traditional"]
            # word["word_type"]
            # word["pinyin"]
            # word["pinyin_numbers"]
            # word["number_of_strokes"]
            # word["sub_components"]
            # word["frequency"]
            # word["hsk_level"]
            # word["top_level"]]
            # word["audio_path"]
            # word["image_path"]
            # word["definition"]

            if not word:
                word = "Not found"
            if not word_traditional:
                word_traditional = "Not found"
            if not pinyin:
                pinyin = "Not found"
            if not definition:
                definition = "Not found"

            note = genanki.Note(
                model=template_decks.CHINESE_CARD_MODEL,
                tags=['autoanki'],
                fields=[word, word_traditional, pinyin, definition],
                # sort_field can be used to sort when the cards appear.
                # By default they are shown in the order they are addeed, so this is not currently used
                sort_field=1,

            )
            self.deck.add_note(note)
            num_of_valid_cards_added += 1

        if not filename.endswith(".apkg"):
            filename += ".apkg"

        genanki.Package(self.deck).write_to_file(filename)
        self.logger.info("Deck " + deck_name + " created with " + str(num_of_valid_cards_added) + " cards")
        return filename




import logging
import os

CLEANED_FILES_DIRECTORY = 'cleaned_files'
CLEANED_FILES_SUFFIX = '_cleaned'
# Sentences That should not be added to the completed file.
GARBAGE_SENTENCES = ['',
                     "",
                     "。",
                     "\n"]


class BookCleaner:

    def __init__(self, debug_level, force=False):
        """ Internal tool used to sanatize input
        Use `clean(bookpath)` to sanatize files and remove junk data
        `force` Ignore confirmations on if you want to clean >50 files
        """
        self.logger = logging.getLogger('autoanki.bookcleaner')
        self.logger.setLevel(debug_level)
        self.file_list = []
        self.bookpath = ""
        self.force = force

    def clean(self, bookpath: str) -> None | list[str]:
        """
        Cleans the files contained in the bookpath. If bookpath is a single file, clean it.
        Args: 
            `bookpath: filepath of files to be cleaned`
        Return: 
            `str: list of cleaned file(s)`
        """
        if not os.path.exists(bookpath):
            self.logger.warning("Cannot find path [" + str(bookpath) + "]")
            return None

        # If the bookpath is a single file, clean and return it
        if os.path.isfile(bookpath):
            cleaned_files = [self._clean_file(bookpath, cleaned_files_root=None)]
            return cleaned_files
        else:
            dirty_files = []
            for root, dirs, files in os.walk(bookpath):
                for file in files:
                    # Only clean files that are not in cleaned_files directory
                    in_cleaned = str(root).find(CLEANED_FILES_DIRECTORY) != -1
                    if not in_cleaned:
                        if '.txt' in file:
                            dirty_files.append(os.path.join(root, file))

            # Check this cleaning won't be mean to the CPU
            if len(dirty_files) > 50 and not self.force:
                yn = input("Over 50 files. Are you sure you want to convert this many files? (Y/N)")
                if yn.lower() != 'y':
                    return None

        # Now we have a list of files to convert in a list
        cleaned_files = []

        # Create directory for files
        cleaned_files_root = os.path.join(bookpath, CLEANED_FILES_DIRECTORY)
        if not os.path.exists(cleaned_files_root):
            os.mkdir(cleaned_files_root)

        for file in dirty_files:
            cleaned_filepath = self._clean_file(file, cleaned_files_root=cleaned_files_root)
            cleaned_files.append(cleaned_filepath)

        return cleaned_files

    def _clean_file(self, filepath, cleaned_files_root):
        """
        Takes a txt file and cleans it up, putting every sentence on a new line
        :param filepath: The txt file to clean
        :param cleaned_files_root: The root
        :return:
        """
        # Set the directory where the cleaned file will go
        if not cleaned_files_root:
            # root/test1.txt -> root/hello_cleaned.txt
            new_filepath = os.path.splitext(filepath)[0] + CLEANED_FILES_SUFFIX + os.path.splitext(filepath)[1]
        else:
            # TODO I'm suspicious that this works every time
            new_filepath = os.path.join(cleaned_files_root, '/'.join(filepath.split('/')[2:]))
        # self.logger.debug(f"Old filepath: [{filepath}]")
        # self.logger.debug(f"New filepath: [{new_filepath}]")

        # Clean page file of characters that may cause issues.
        page_file = open(filepath, encoding='utf-8')
        try: 
            page_sentences = page_file.read().split("。")
        except:
            self.logger.error(f"Critical error cleaning file: [{filepath}]")
            return
        cleaned_file = open(new_filepath, "w", encoding='utf-8')

        for page_sentence in page_sentences:
            # Clean string
            page_sentence = page_sentence\
                .lstrip()\
                .rstrip()\
                .replace("  ", " ")\
                .lstrip("\"")\
                .rstrip("\"")\
                .replace("”","'")\
                .replace("　", "")\
                .replace("“","")\
                .rstrip("。")\
                .strip("'")\
                .strip("'")
            if page_sentence not in GARBAGE_SENTENCES:
                cleaned_file.write(page_sentence + "。" + "\n")

        return new_filepath




import jieba
import chinese_converter
from string import punctuation
import logging

from autoanki.Dictionary.CEDictionary import CEDictionary

# TODO Is there a way to do this in a smarter way? Maybe check if the characters are in a certian UTF-8 block?
PUNCTUATION = """
+,;:'()[]{}&*^%$#@!◇♦•·■◎∞=™©×
"""
CHINESE_PUNC = "！？｡。．.…、＂＃＄％＆＇（）＊＋，－／：；＜＝＞＠［＼］＾＿｀｛｜｝～｟｠｢｣､、〃《》「」『』【】〔〕〖〗〘〙〚〛〜〝〞〟〰〾〿–—‘’‛“”„‟…‧﹏  ① ② ⑽ "

# TODO: This can be extened and implemented
OTHER = "ｗ９ｌｉｔｂｎｅｐｈⅠ Ⅱ Ⅲ Ⅳ "

# TODO Remove straight numbers and english words

class ChineseTokenizer():

    def __init__(self, debug_level=20, dictionary=None):
        self.logger = logging.getLogger('autoanki.dbmngr')
        self.logger.setLevel(debug_level)

        # Currently, each element tokenized will be checked against the dictionary, 
        #   and shortented if there is not a match
        # TODO It's silly how much processing power this might use in worst-case. 
        #   Can we improve on this?
        if dictionary:
            self.dictionary = dictionary
        else:
            self.dictionary = CEDictionary(debug_level=20)

    def tokenize(self, line:str) -> None | list[str]:
        
        # TODO jieba has more features that we should take advantage of to get better tokenizing
        dirty_words = jieba.lcut(line)
        clean_words = []
        self.logger.debug("Dirty words:")
        self.logger.debug(dirty_words)
        for word in dirty_words:
            word = word.strip('\n')
            if not word:
                continue
            # Convert the word to simplified if needed
            word = chinese_converter.to_simplified(word)
            if len(word) == 1:
                if word in PUNCTUATION:
                    continue
                if word in CHINESE_PUNC:
                    continue
                if word in OTHER:
                    continue

            # If there is a definition in the dictionary, skip any more processing
            if not word:
                continue
            if self.dictionary.find_word(word):
                clean_words.append(word)
                continue


            # Try tokenizing again?
            subwords = jieba.lcut(word)
            if len(subwords) > 1:
                for subword in subwords:
                    clean_words.append(subword)
                continue

            # Remove all ascii
            new_word = ''.join(i for i in word if ord(i)>128)
            # if word != new_word:
                # self.logger.info(f"{word} -> {new_word}", )
            word = new_word
            if not word:
                continue
            if self.dictionary.find_word(word):
                clean_words.append(word)
                continue

            word = self.remove_modifiers(word)
            if not word:
                continue
            if self.dictionary.find_word(word):
                clean_words.append(word)
                continue

            word = self.remove_numbers(word)
            if not word:
                continue
            if self.dictionary.find_word(word):
                clean_words.append(word)
                continue

            # Repeated characters (always?) contain the same meaning as one, just varied slightly
            # 人人 = everyone
            if len(word) == 2 and word[0] == word[1]:
                word = word[0]
            if not word:
                continue
            if self.dictionary.find_word(word):
                clean_words.append(word)
                continue

            # TODO 2 repeated, 1?
            #   点点头
            #   长长的
            # TODO 的 at the end?
            # TODO 2 repeated, 2 repeated?
            #   起起伏伏
            # TODO Some gramatical patterns?
            # 在。。。上

            if len(word) == 2:
                clean_words.append(word[0])
                clean_words.append(word[1])
                continue

            if len(word) == 3:
                clean_words.append(word[0])
                clean_words.append(word[1])
                clean_words.append(word[2])
                continue

            if len(word) == 4:
                if self.dictionary.find_word(word[0:1]):
                    clean_words.append(word[0:1])
                else:
                    clean_words.append(word[0])
                    clean_words.append(word[1])

                if self.dictionary.find_word(word[2:3]):
                    clean_words.append(word[2:3])
                else:
                    clean_words.append(word[2])
                    clean_words.append(word[3])
                continue

            if len(word) == 5:
                clean_words.append(word[0])
                clean_words.append(word[1])
                clean_words.append(word[2])
                clean_words.append(word[3])
                clean_words.append(word[4])
                continue

            clean_words.append(word)

        if clean_words:
            self.logger.debug("Clean words")
            self.logger.debug(clean_words)
        return clean_words 


    def remove_numbers(self, word:str):
        # Remove all numbers from the front
        # Lots of the words follow the following format:
        #   Number + Subject
        CHINESE_NUMBERS = "第一二两三四五五六七八九十百千万满"
        old_word = "" 
        temp_word = word
        while old_word != temp_word:
            old_word = temp_word 
            if len(temp_word) == 0:
                break
            if temp_word[0] in CHINESE_NUMBERS:
                temp_word = temp_word[1:]
        return temp_word

    def remove_modifiers(self, word:str):
        stripped_word = word.lstrip('小')
        stripped_word = stripped_word.lstrip('大')
        stripped_word = stripped_word.lstrip('这')
        stripped_word = stripped_word.lstrip('那')
        stripped_word = stripped_word.lstrip('不')
        stripped_word = stripped_word.lstrip('几')
        stripped_word = stripped_word.lstrip('无')
        stripped_word = stripped_word.lstrip('没')
        stripped_word = stripped_word.lstrip('全')
        stripped_word = stripped_word.lstrip('上')
        stripped_word = stripped_word.lstrip('下')
        stripped_word = stripped_word.lstrip('太')
        stripped_word = stripped_word.lstrip('一个')
        return stripped_word





import logging
import time
import urllib

import requests
import selenium
from selenium import webdriver
from bs4 import BeautifulSoup
import urllib.parse
import pinyin as pin_to_num

from autoanki.Dictionary.Dictionary import Dictionary

class YellowBridgeDictionary(Dictionary):

    def __init__(self, debug_level):
        self.logger = logging.getLogger('autoanki.ybdict')
        self.logger.setLevel(debug_level)
        self.logger.debug(f"logger active")
        self.logger.info("Loading Yellowbridge...")

    def find_word(self, word, cache_number=None):
        '''
        Helper function for complete_unfinished_dictionary_records() Takes a word (one or more characters),
        finds them on YellowBridge, and adds them to the dictionary
        :param word: The word to find on YellowBridge.
        :param cache_number: The secondary page number for a word with multiple definitions. (See comments in body of
        function)
        :return:
        '''
        if word == None or word == '':
            return

        self.logger.debug("Finding: " + repr(word))

        urlx = "http://www.yellowbridge.com/chinese/dictionary.php?word="
        url = urlx + urllib.parse.quote(word)
        if cache_number is not None:
            url += "&cache=" + cache_number

        driver = webdriver.Chrome()
        response = driver.get(url)
        html = driver.page_source
        # response = requests.get(url, headers={ 'user-agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.110 Safari/537.36'})
        yellowbridge_soup = BeautifulSoup(html, "html.parser")
        main_data = yellowbridge_soup.find(id='mainData')
        self.logger.debug(yellowbridge_soup)

        # Some definitions will show up on the page differently because they have multiple meanings/pronunciations
        # These pages link to other pages with a different 'cache' number
        # Default 了 page:
        # https://www.yellowbridge.com/chinese/dictionary.php?word=%E4%BA%86
        # Page for le:
        # https://www.yellowbridge.com/chinese/dictionary.php?word=%E4%BA%86&cache=5114
        # Page for laio:
        # https://www.yellowbridge.com/chinese/dictionary.php?word=%E4%BA%86&cache=5115
        # To solve this, return `find_word()` with the first cache number
        if not main_data:
            self.logger.debug("Main data not found. Multiple definitions")
            # print("This is not a definition page. getting all sub-pages")
            matching_results = yellowbridge_soup.find(id='multiRow')
            # Grab first row. This is usually the most common definition
            rows = matching_results.find_all('tr')
            if not rows:
                self.logger.error("No rows found ")
                return None
            # print(row.find('a'))
            href = str(row.find('a')['href'])
            # print(href)
            cache_number_from_href = href.split("word=")[1].split("&")[1].replace("cache=", "")

            return self.find_word(word, cache_number_from_href)

        definition = \
        traditional_script = \
        pinyin = \
        pinyin_num = \
        part_of_speech = \
        hsk_level = \
        top_level = \
        composing_words = None

        main_data = main_data.find_all('tr')

        # Collect all necessary information from the page:
        for row in main_data:
            row_info_type = row.find('td').getText()
            row_info = row.find_all('td')[1].getText()
            # print(row_info_type + ":")
            # print(row_info)
            if row_info_type == "English Definition":
                definition = row_info
            elif row_info_type == "Simplified Script":
                simplified_script = row_info
            elif row_info_type == "Traditional Script":
                traditional_script = row_info.split("P")[0]
            elif row_info_type == "Pinyin":
                pinyin = row_info
                pinyin_num = pin_to_num.get(word, format="numerical")
            elif row_info_type == "Part of Speech":
                part_of_speech = row_info
            elif row_info_type == "Proficiency Test Level":
                proficiency_level = row_info
                if "HSK=" in proficiency_level:
                    hsk_level = proficiency_level.split("HSK=")[1].split(";")[0]
                if "TOP=" in proficiency_level:
                    top_level = proficiency_level.split("TOP=")[1].split(";")[0]

        word_decomposition_soup = yellowbridge_soup.find(id='wordDecomp')
        if len(word) > 1:
            composing_words_list = word_decomposition_soup.find_all('tr')
            composing_words = ""
            for composing_word in composing_words_list:
                composing_words += str(composing_word.find_all('td')[0].find('a').getText()) + ";"

        # This is all of the information that has been collected
        # print([definition, simplified_script, traditional_script,pinyin,pinyin_num, part_of_speech,hsk_level,top_level,composing_words])
        params = [traditional_script, part_of_speech, pinyin, pinyin_num,
                  composing_words, hsk_level, top_level, definition, word]
        # Add the information for this word to the database
        return params

    def size(self):
        return 0




import logging

from autoanki.BookCleaner import BookCleaner
from autoanki.DatabaseManager import DatabaseManager
from autoanki.Dictionary import CEDictionary
from autoanki.DeckManager import DeckManager
from autoanki.Tokenizer import ChineseTokenizer

BLACK = "\u001b[30m"
RED = "\u001b[31m"
GREEN = "\u001b[32m"
YELLOW = "\u001b[33m"
BLUE = "\u001b[34m"
MAGENTA = "\u001b[35m"
CYAN = "\u001b[36m"
WHITE = "\u001b[37m"
RESET = "\u001b[0m"
logging.basicConfig(
    # filename='HISTORY.log',
    level=logging.WARNING,
    format=f'{GREEN}%(asctime)s{RESET} {RED}%(levelname)8s{RESET} {YELLOW}%(name)16s{RESET}: %(message)s',
    datefmt='%Y-%m-%d %H:%M:%S',
)

class AutoAnki:

    def __init__(self, database_filepath='autoanki.db', debug_level=20, force=False, dictionary=None):
        """
        Creates an instance of autoanki.
        This creates a book cleaner, database connection, dictioary connection, and deck maker
        :param database_filepath: The filepath for the database
        :param logging_level: between 0 (DEBUG) and 50(CRITICAL)
        :param `force`: Skip conformations for cleaning large numbers of files
        """
        self.logger = logging.getLogger('autoanki')
        self.logger.setLevel(debug_level)
        self.logger.debug(f"logger active")
        self.logger.info("Connecting to database...")

        self.force = force
        self.book_cleaner = BookCleaner(debug_level, self.force)

        if dictionary:
            self.logger.info("Using custom dictionary")
        else:
            self.dictionary = CEDictionary(debug_level)

        self.database_filepath = database_filepath
        if not DatabaseManager.is_database(database_filepath):
            self.logger.info("Creating database...")
            DatabaseManager.create_database(database_filepath)
            self.logger.info("Done creating database.")
        self.database_manager = DatabaseManager(database_filepath, debug_level)

        self.deck_manager = DeckManager(debug_level)

        self.logger.info("Connected!")

    def add_book(self, book_path: str, book_name: str = 'New Book'):
        """
        Add a directory full of files to the database
        :param book_path: The filepath to the directory that contains the files to add. e.g. lost_prince.txt
        :param book_name: The name of the book being added e.g. "Lost Prince"
        :return:
        """
        self.logger.debug(f"autoanki: Adding book from [{book_path}]")

        # Clean the book
        if not self.book_cleaner.clean(book_path):
            # self.logger.warning("autoanki: Unable to clean book [" + book_name + "].")
            return

        # Add the book to the database
        if not self.database_manager.add_book(book_path, book_name):
            self.logger.warning("Unable to add [" + book_name + "] to database.")
            return

        self.logger.info("autoanki: Added [" + book_path + "].")

    def complete_unfinished_definitions(self):
        """
        autoanki contains an internal definitions table that is scraped from the internet. As words are added to
        autoanki, their definitions must be found.
        This function finds definitions and adds them to the table
        """
        # TODO Make progress bar for unfinished records
        self.logger.info("Checking for records...")
        self.database_manager.cursor.execute("SELECT word FROM dictionary WHERE definition IS NULL")
        response_rows = self.database_manager.cursor.fetchall()
        if len(response_rows) == 0:
            self.logger.info("No new rows to complete in dictionary table")
            return

        self.logger.info("Adding " + str(len(response_rows)) + " rows to dictionary table")
        self.tokenizer = ChineseTokenizer()
        for row in response_rows:
            word = str(row[0])

            # self.logger.debug(f"Finding: [{word}]")
            # self.logger.debug("Trying local dictionary...")
            params = self.dictionary.find_word(word)
            if params:
                # self.logger.debug(f"✅Found: [{params[8]}]")
                self.database_manager.update_definition(params)
                continue

            self.logger.info(f"❌Could not find: [{word}]")

    def deck_settings(self,
                      inclue_traditional = True,
                      inclue_part_of_speech = True,
                      word_frequency_filter = None,
                      ):
        """Configures settings for what's in the deck, and how it looks"

        `word_frequency_filter`: Float between 0 and 1. 1 being every word is included, 0 being none are included
        """
        self.deck_manager.settings(
            inclue_traditional,
            inclue_part_of_speech,
            word_frequency_filter,
        )

    def print_database_info(self):
        self.database_manager.print_info()

    @staticmethod
    def is_database(db_path):
        return DatabaseManager.is_database(db_path)

    @staticmethod
    def create_database(db_path: str):
        DatabaseManager.create_database(db_path)

    def create_deck(self, deck_name: str, filepath: str):
        """
        Creates a deck file in the directory of the main file.
        `deck_name` The name that will show up in Anki
        `filepath` Path to the file
        :return:
        """

        self.logger.info("Generating deck file [" + deck_name + ".apk]")
        words = self.database_manager.get_all_completed_definitions()

        deck_path = self.deck_manager.generate_deck_file(words, deck_name, filepath)
        if deck_path is None:
            self.logger.warning("Was not able to create deck file for [", deck_name, "]")
        else:
            self.logger.info("Generated deck file [" + deck_path + "]")

    @property
    def book_list(self):
        """
        Get a list of the books in the database
        :return: List of book names
        """
        return self.database_manager.books

    @book_list.setter
    def book_list(self, _):
        pass

    @property
    def unfinished_entries(self):
        return self.database_manager.unfinished_definitions()

    @unfinished_entries.setter
    def unfinished_entries(self, _):
        pass


